@@ -1,33 +1,35 @@
 package Class::PObject::Driver::DBI;
 
-# $Id: DBI.pm,v 1.3 2003/08/23 10:36:44 sherzodr Exp $
+# $Id: DBI.pm,v 1.8 2003/09/02 22:22:02 sherzodr Exp $
 
 use strict;
 use Carp;
+use Log::Agent;
 use Class::PObject::Driver;
+use Data::Dumper;
 use vars ('$VERSION', '@ISA');
 
 @ISA = ('Class::PObject::Driver');
-$VERSION = '1.00';
+
+$VERSION = '2.00';
 
 
 sub _prepare_where_clause {
     my ($self, $terms) = @_;
 
     $terms ||= {};
-
+    unless ( ref $terms ) {
+        die join ', ', caller(0)
+    }
     # if no terms present, just return an empty string
     unless ( keys %$terms ) {
         return ("", ());
     }
-
     my ($sql, @where, @bind_params);
-
     while ( my ($k, $v) = each %$terms ) {
         push @where, "$k=?";
         push @bind_params, $v
     }
-
     $sql = "WHERE " . join (" AND ", @where);
     return ($sql, \@bind_params)
 }
@@ -35,13 +37,12 @@ sub _prepare_where_clause {
 
 
 sub _prepare_select {
-    my ($self, $table_name, $terms, $args) = @_;
-
-    my ($sql, @where, @bind_params);
+    my ($self, $table_name, $terms, $args, $cols) = @_;
 
+    my ($sql, @where, @bind_params, $selected_cols);
     my ($where_clause, $bind_params) = $self->_prepare_where_clause($terms);
-    $sql = "SELECT * FROM $table_name " . $where_clause;
-
+	$selected_cols = $cols ? join (", ", @$cols) : "*";
+    $sql = sprintf("SELECT %s FROM %s %s", $selected_cols, $table_name, $where_clause);
     if ( defined $args ) {
         $args->{limit}      ||= 1000;
         $args->{offset}     ||= 0;
@@ -133,22 +134,40 @@ sub _tablename {
 
 
 
-
-
-
 sub load {
+    my $self = shift;
+    my ($object_name, $props, $id) = @_;
+
+    my $dbh = $self->dbh($object_name, $props) or return;
+    my $table = $self->_tablename($object_name, $props, $dbh) or return;
+    my ($sql, $bind_params) = $self->_prepare_select($table, {id=>$id});
+
+    my $sth = $dbh->prepare( $sql );
+    unless ( $sth->execute( @$bind_params ) ) {
+        $self->errstr($sth->errstr);
+        return undef
+    }
+
+    return $sth->fetchrow_hashref
+}
+
+
+
+
+
+
+
+sub load_ids {
     my $self = shift;
     my ($object_name, $props, $terms, $args) = @_;
 
-    if ( $terms && (ref($terms) ne 'HASH') && ($terms =~m/^\d+$/) ) {
-        $terms = {id => $_[2]}
-    }
-
-    my $dbh   = $self->dbh($object_name, $props)                  or return;
+    my $dbh   = $self->dbh($object_name, $props)              or return;
     my $table = $self->_tablename($object_name, $props, $dbh) or return;
-    my ($sql, $bind_params)   = $self->_prepare_select($table, $terms, $args);
-
+    my ($sql, $bind_params)   = $self->_prepare_select($table, $terms, $args, ['id']);
+    
+    
     my $sth   = $dbh->prepare( $sql );
+    
     unless( $sth->execute(@$bind_params) ) {
         $self->errstr($sth->errstr);
         return undef
@@ -158,11 +177,11 @@ sub load {
         return []
     }
 
-    my @rows = ();
+    my @data_set = ();
     while ( my $row = $sth->fetchrow_hashref() ) {
-        push @rows, $row
+        push @data_set, $row->{id}
     }
-    return \@rows
+    return \@data_set
 }
 
 
@@ -230,11 +249,61 @@ sub count {
         return undef
     }
 
-    return $sth->fetchrow_array || 0
+    my $count = $sth->fetchrow_array || 0;
+    return $count
 }
 
 
 
+
+
+
+sub _read_lock {
+    my ($self, $dbh, $table) = @_;
+
+
+
+
+
+
+}
+
+
+
+
+
+
+
+sub _write_lock {
+    my ($self, $dbh, $table) = @_;
+
+
+
+
+
+
+}
+
+
+
+
+
+sub _unlock {
+    my ($self, $dbh, $table) = @_;
+
+
+
+
+
+
+}
+
+
+
+
+
+
+
 1;
 __END__
 # Below is stub documentation for your module. You'd better edit it!
