@@ -1,14 +1,19 @@
 package Class::PObject::Template;
 
-# $Id: Template.pm,v 1.7 2003/08/23 10:36:40 sherzodr Exp $
+# $Id: Template.pm,v 1.16 2003/08/28 16:32:24 sherzodr Exp $
 
 use strict;
 use Log::Agent;
+use vars ('$VERSION');
+
+$VERSION = '1.02';
 
 sub new {
     my $class = shift;
     $class    = ref($class) || $class;
 
+    logtrc 2, "%s->new(%s)", $class, join ", ", @_;
+
     # What should be done if we detect odd number of arguments?
     # I'd say we should croak() right away. We don't want to
     # end-up with corrupted record in the database, whether the
@@ -20,43 +25,29 @@ sub new {
     my $props = $class->__props();
 
     # object properties as represented internally by Class::PObject.
-    # Whoever accesses this information from within their code will be shot
+    # Whoever accesses this information from within their code should be shot
     my $self = {
-        columns     => { },   # <-- holds key/value pairs
+        columns     => { @_ },   # <-- holds key/value pairs
         _is_new     => 1
     };
 
-    # DBD::CVS seems to keep all the column names in uppercase. This is a problem,
-    # when the load() method calls new() with key/value pairs while creating an object
-    # off the disk. So we first convert the @_ to a hashref
-    my $args = { @_ };
-    # and fill in 'columns' attribute of the class with their lower-cased names:
-    while ( my ($k, $v) = each %$args ) {
-        $self->{columns}->{lc $k}   = $v
-    }
+	bless($self, $class);
 
     # It's possible that new() was not given all the column/values. So we
     # detect the ones missing, and assign them 'undef'
     for my $colname ( @{$props->{columns}} ) {
-        $self->{columns}->{$colname} ||= undef
+        unless ( defined $self->{columns}->{$colname} ) {
+            $self->{columns}->{$colname} = undef
+        }
     }
 
-    # I'm not sure if 'datasource' should be mandatory. I'm open to
-    # any suggestions. Some drivers may be able to recover 'datasource' on the fly,
-    # without asking for expclicit definition.
-    #unless ( defined $self->{datasource} ) {
-        #$class->error("'datasource' is required");
-        #return undef;
-    #}
-
     # we may also check if the driver is indeed a valid one. However doing so
     # does not allow creating in-memory objects without valid driver. So let's leave
     # this test for related methods.
 
-    bless($self, $class);
-
-    # if _init() has been specified, we shoudl call it.
+    # if pobject_init() exists, we should call it
     if ( $self->UNIVERSAL::can('pobject_init') ) {
+        logtrc 2, "calling pobject_init()";
         $self->pobject_init
     }
     return $self
@@ -69,6 +60,8 @@ sub save {
     my $self  = shift;
     my $class = ref($self) || $self;
 
+    logtrc 2, "%s->save(%s)", $class, join ", ", @_;
+
     my $props = $self->__props();
     my $driver_obj = $self->__driver();
 
@@ -89,13 +82,45 @@ sub save {
 
 
 
+
+sub fetch {
+    my $self = shift;
+    my ($terms, $args) = @_;
+    my $class = ref($self) || $self;
+
+    logtrc, "%s->fetch(%s)", $class, join ", ", @_;
+
+    $terms ||= {};
+    $args  ||= {};
+    
+    my $props  = $self->__props();
+    my $driver = $self->__driver();
+
+    my $ids = $driver->load_ids($class, $props, $terms, $args);
+
+    require Class::PObject::Iterator;
+    return Class::PObject::Iterator->new($class, $ids);
+}
+
+
+
+
+
+
+
+
 sub load {
     my $self  = shift;
+    my ($terms, $args) = @_;
     my $class = ref($self) || $self;
 
+    logtrc 2, "%s->load(%s)", $class, join ", ", @_;
+
+    $terms ||= {};
+    $args  ||= {};
+
     # if we're called in void context, why bother?
     unless ( defined wantarray() ) {
-        logtrc 2, "load() called in void context";
         return undef
     }
 
@@ -107,18 +132,28 @@ sub load {
 
     my $props       = $self->__props();
     my $driver_obj  = $self->__driver();
+    my $ids         = [];       # we first initialize an empty ID list
 
-    my $rows        = $driver_obj->load($class, $props, @_) or return;
-    unless ( scalar @$rows ) {
-        $self->errstr( $driver_obj->errstr );
+    # now, if we had a single argument, and that argument was not a HASH,
+    # we assume we received an ID
+    if ( $terms && (ref($terms)  ne 'HASH') && ($terms =~ /^\d+$/) ) {
+        $ids = [ $terms ]
+
+    } else {
+        $ids        = $driver_obj->load_ids($class, $props, $terms, $args) or return
+
+    }
+
+    unless ( scalar @$ids ) {
         return ()
     }
 
     # if called in array context, we return an array of objects:
     if (  wantarray() ) {
         my @data_set = ();
-        for ( @$rows ) {
-            my $o = $self->new(%$_);
+        for my $id ( @$ids ) {
+            my $row = $driver_obj->load($class, $props, $id) or next;
+            my $o = $self->new(%$row);
             $o->{_is_new} = 0;
             push @data_set, $o
         }
@@ -126,7 +161,8 @@ sub load {
     }
 
     # if we come this far, we're being called in scalar context
-    my $o = $self->new( %{ $rows->[0] } );
+    my $row = $driver_obj->load($class, $props, $ids->[0]) or return;
+    my $o = $self->new( %$row );
     $o->{_is_new} = 0;
     return $o
 }
@@ -137,6 +173,7 @@ sub remove {
     my $self    = shift;
     my $class   = ref $self;
 
+    logtrc 2, "%s->remove()", $class;
     unless ( ref $self ) {
         logcroak "remove() used as a static method";
     }
@@ -168,6 +205,8 @@ sub remove_all {
     my $self = shift;
     my $class = ref($self) || $self;
 
+    logtrc 2, "%s->remove_all(%s)", $class, join ", ", @_;
+
     my $props = $self->__props();
     my $driver_obj = $self->__driver();
 
@@ -188,6 +227,8 @@ sub count {
     my $self = shift;
     my $class = ref($self) || $self;
 
+    logtrc 2, "%s->count(%s)", $class, join ", ", @_;
+
     my $props      = $self->__props();
     my $driver_obj = $self->__driver();
 
@@ -211,7 +252,12 @@ sub errstr {
 
 
 sub columns {
-    return $_[0]->{columns}
+    my $self = shift;
+    my $class = ref($self) || $self;
+
+    logtrc 2, "%s->columns()", $class;
+
+    return $self->{columns}
 }
 
 
